import React from "react";
import { DatePickerProps, DateTimePickerProps, LocalizationProviderProps } from "@mui/x-date-pickers";
import { AutocompleteProps, FormControlProps, SelectProps, TextFieldProps } from "@mui/material";
import { GridValueOptionsParams } from "@mui/x-data-grid";
import { ValueOption } from "../types";
export declare type ExternalBuilderProps<TDate = any> = {
    searchMenuItems?: ({
        label: string;
        onClick: () => void;
    })[];
    onSubmit?: (params: FilterParameters) => (void | any);
    onRestoreState?: (params: FilterParameters, state?: any) => void;
    localeText?: FilterBuilderLocaleText;
    autocompleteGroups?: string[];
    autocompleteProps?: AutocompleteProps<any, any, any, any>;
    datePickerProps?: DatePickerProps<string, TDate>;
    dateTimePickerProps?: DateTimePickerProps<string, TDate>;
    localizationProviderProps?: LocalizationProviderProps;
    selectProps?: SelectProps;
    textFieldProps?: TextFieldProps;
    disableHistory?: boolean;
    filter?: SerialisedGroup;
};
export declare type FilterParameters = {
    compute?: string;
    filter: string;
    queryString?: QueryStringCollection;
    select?: string[];
    serialised?: SerialisedGroup;
};
export declare type FilterBuilderLocaleText = {
    and?: string;
    or?: string;
    addCondition?: string;
    addGroup?: string;
    field?: string;
    operation?: string;
    value?: string;
    collectionOperation?: string;
    collectionField?: string;
    search?: string;
    reset?: string;
    opAny?: string;
    opAll?: string;
    opCount?: string;
    opEq?: string;
    opNe?: string;
    opGt?: string;
    opLt?: string;
    opGe?: string;
    opLe?: string;
    opContains?: string;
    opNull?: string;
    opNotNull?: string;
};
export declare type BaseFieldDef<TDate> = {
    field: string;
    autocompleteGroup?: string;
    caseSensitive?: boolean;
    datePickerProps?: DatePickerProps<string, TDate>;
    dateTimePickerProps?: DateTimePickerProps<string, TDate>;
    filterable?: boolean;
    filterField?: string;
    filterOperators?: Operation[];
    filterType?: string;
    getCustomFilterString?: (op: Operation, value: any) => string | FilterCompute | boolean;
    getCustomQueryString?: (op: Operation, value: any) => QueryStringCollection;
    label?: string;
    nullable?: boolean;
    selectProps?: {
        selectProps?: SelectProps;
        formControlProps?: FormControlProps;
        label?: string;
    };
    sortField?: string;
    textFieldProps?: TextFieldProps;
    renderCustomInput?: (value: any, setValue: (v: any) => void) => React.ReactNode;
    renderCustomFilter?: (value: any, setValue: (v: any) => void) => React.ReactNode;
    type?: string;
    valueOptions?: ValueOption[] | ((params: GridValueOptionsParams) => ValueOption[]);
};
export declare type FieldDef<TDate = any> = BaseFieldDef<TDate> & {
    headerName?: string;
    collection?: boolean;
    collectionFields?: CollectionFieldDef<TDate>[];
};
export declare type CollectionFieldDef<TDate = any> = BaseFieldDef<TDate>;
export declare type FilterCompute = {
    filter: string;
    compute: string | ComputeSelect;
};
export declare type ComputeSelect = {
    compute: string;
    select: string[];
};
export declare type QueryStringCollection = {
    [key: string]: string;
};
export declare type FilterTranslatorCollection<TDate> = {
    [key in Operation | "default"]?: FilterTranslator<TDate>;
};
export declare type FilterTranslator<TDate> = (params: FilterTranslatorParams<TDate>) => string | boolean;
export declare type FilterTranslatorParams<TDate> = {
    schema: BaseFieldDef<TDate>;
    field: string;
    op: Operation;
    value: any;
};
export declare type Connective = "and" | "or";
export declare type Operation = "eq" | "ne" | "gt" | "lt" | "ge" | "le" | "contains" | "null" | "notnull";
export declare type CollectionOperation = "any" | "all" | "count";
declare type Clause = {
    id: string;
};
export declare type GroupClause = Clause & {
    connective: Connective;
};
export declare type ConditionClause = Clause & {
    field: string;
    op: Operation;
    collectionOp?: CollectionOperation;
    collectionField?: string;
    value: any;
    default?: boolean;
};
export declare type TreeGroup = Clause & {
    children: TreeChildren;
};
export declare type TreeChildren = Immutable.Map<string, TreeGroup | string>;
export declare type StateClause = Immutable.Map<string, GroupClause | ConditionClause>;
export declare type StateTree = Immutable.Map<string, string | TreeGroup>;
export declare type SerialisedGroup = Omit<GroupClause, "id"> & {
    children: (SerialisedGroup | SerialisedCondition)[];
};
export declare type SerialisedCondition = Omit<ConditionClause, "id" | "default">;
export {};
