import { ResponsiveValues } from "./hooks";
import { ExternalBuilderProps, FieldDef } from "./FilterBuilder/types";
import React from "react";
export declare type ODataGridBaseProps<ComponentProps extends IGridProps, SortModel extends IGridSortModel, ColDef, TRow, TDate> = OmitGridProps<ComponentProps> & {
    url: string;
    alwaysSelect?: string[];
    columns: ODataGridBaseColDef<ColDef, TDate>[];
    columnVisibilityModel?: ODataColumnVisibilityModel;
    component: React.ElementType;
    defaultPageSize?: number;
    defaultSortModel?: SortModel;
    disableFilterBuilder?: boolean;
    disableHistory?: boolean;
    $filter?: string;
    filterBuilderProps?: ExternalBuilderProps<TDate>;
    requestOptions?: RequestInit;
};
declare type OmitGridProps<T> = Omit<T, "columns" | "rows" | "autoPageSize" | "columnVisibilityModel" | "disableColumnFilter" | "filterMode" | "filterModel" | "loading" | "onFilterModelChange" | "onPageChange" | "onPageSizeChange" | "page" | "pageSize" | "paginationMode" | "rowCount" | "sortingMode" | "sortModel">;
declare type ODataColumn<T, TDate> = Omit<T, "filterOperators" | "hide" | "sortComparator"> & FieldDef<TDate> & {
    select?: string;
    expand?: Expand | Expand[];
    hide?: ResponsiveValues<boolean> | boolean;
    filterOnly?: boolean;
};
export declare type ODataRowModel<T> = {
    result: T;
    [key: string]: any;
};
export declare type ODataGridBaseColDef<ColDef, TDate> = ODataColumn<ColDef, TDate>;
export declare type ODataGridBaseEnrichedColDef<ColDef, ActionsColDef, TDate> = ODataColumn<ColDef, TDate> | ODataColumn<ActionsColDef, TDate>;
export declare type ODataBaseGridColumns<EnrichedColDef, ActionsColDef, TDate> = ODataGridBaseEnrichedColDef<EnrichedColDef, ActionsColDef, TDate>[];
export declare type ODataResponse<T> = {
    "@odata.count"?: number;
    value: T[];
};
export declare type Expand = {
    navigationField: string;
    select?: string;
    expand?: Expand[] | Expand;
    orderBy?: string;
    top?: number;
    count?: boolean;
};
export declare type ValueOption = string | number | SelectOption;
export declare type SelectOption = {
    value: any;
    label: string;
};
export declare type ODataColumnVisibilityModel = Record<string, boolean | ResponsiveValues<boolean>>;
export declare type ColumnVisibilityModel = Record<string, boolean>;
export declare type IGridSortModel = ({
    field: string;
    sort: 'asc' | 'desc' | null | undefined;
})[];
export declare type IGridRowModel<T = {
    [key: string]: any;
}> = T;
export declare type IGridProps = {
    onColumnVisibilityModelChange?: any;
    columnVisibilityModel?: ColumnVisibilityModel;
    onSortModelChange?: any;
};
export {};
