'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jsxRuntime = require('react/jsx-runtime');
var React = require('react');
var system = require('@mui/system');
var styles = require('@mui/material/styles');
var recoil = require('recoil');
var iconsMaterial = require('@mui/icons-material');
var material = require('@mui/material');
var Immutable = require('immutable');
var xDatePickers = require('@mui/x-date-pickers');
var uuid = require('uuid');
var tssReact = require('tss-react');
var xDataGridPro = require('@mui/x-data-grid-pro');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var Immutable__default = /*#__PURE__*/_interopDefaultLegacy(Immutable);

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const useResponsive = () => {
    const theme = styles.useTheme();
    const matches = useBreakpoints();
    return function (responsiveValues) {
        let match;
        theme.breakpoints.keys.forEach((breakpoint) => {
            if (matches[breakpoint] && responsiveValues[breakpoint] != null) {
                match = breakpoint;
            }
        });
        return match && responsiveValues[match];
    };
};
// eslint-disable-next-line react-hooks/exhaustive-deps
const useMountEffect = (func) => React.useEffect(func, []);
const useBreakpoints = () => {
    const theme = styles.useTheme();
    const [matches, setMatches] = React.useState(getMatches(theme.breakpoints.keys, theme));
    React.useEffect(() => {
        const queries = getQueries(theme.breakpoints.keys, theme);
        const listeners = {};
        const updateMatch = (b) => {
            setMatches((oldMatches) => { var _a, _b; return (Object.assign(Object.assign({}, oldMatches), { [b]: (_b = (_a = queries[b]) === null || _a === void 0 ? void 0 : _a.matches) !== null && _b !== void 0 ? _b : false })); });
        };
        theme.breakpoints.keys.forEach(b => {
            listeners[b] = () => updateMatch(b);
            queries[b].addEventListener("change", listeners[b]);
        });
        return () => {
            theme.breakpoints.keys.forEach(b => {
                queries[b].removeEventListener("change", listeners[b]);
            });
        };
    }, [theme]);
    return matches;
};
const getQueries = (breakpoints, theme) => breakpoints.reduce((acc, b) => (Object.assign(Object.assign({}, acc), { [b]: window.matchMedia(theme.breakpoints.up(b).replace(/^@media( ?)/m, '')) })), {});
const getMatches = (breakpoints, theme) => breakpoints.reduce((acc, b) => (Object.assign(Object.assign({}, acc), { [b]: window.matchMedia(theme.breakpoints.up(b).replace(/^@media( ?)/m, '')).matches })), {});

const rootGroupUuid = "17c63a07-397b-4f03-a74b-2f935dcc6c8a";
const rootConditionUuid = "18c1713a-2480-40c0-b60f-220a3fd4b117";
const allOperators = ["eq", "ne", "gt", "lt", "ge", "le", "contains", "null", "notnull"];
const numericOperators = ["eq", "ne", "gt", "lt", "ge", "le"];
const initialClauses = Immutable__default["default"].Map({
    [rootGroupUuid]: {
        id: rootGroupUuid,
        connective: "and"
    },
    [rootConditionUuid]: {
        id: rootConditionUuid,
        field: "",
        op: "eq",
        value: null,
        default: true
    }
});
const initialTree = Immutable__default["default"].Map({
    [rootGroupUuid]: {
        id: rootGroupUuid,
        children: Immutable__default["default"].Map({ [rootConditionUuid]: rootConditionUuid })
    }
});
const defaultLocale = {
    and: "And",
    or: "Or",
    addCondition: "Add Condition",
    addGroup: "Add Group",
    field: "Field",
    operation: "Operation",
    value: "Value",
    collectionOperation: "Operation",
    collectionField: "Field",
    search: "Search",
    reset: "Reset",
    opAny: "Has at least one",
    opAll: "All have",
    opCount: "Count",
    opEq: "=",
    opNe: "≠",
    opGt: ">",
    opLt: "<",
    opGe: "≥",
    opLe: "≤",
    opContains: "Contains",
    opNull: "Is Blank",
    opNotNull: "Is Not Blank"
};

const schemaState = recoil.atom({
    key: "schema",
    default: []
});
const clauseState = recoil.atom({
    key: "filterClauses",
    default: initialClauses
});
const treeState = recoil.atom({
    key: "filterTree",
    default: initialTree
});
const propsState = recoil.atom({
    key: "props",
    default: {}
});

const getDefaultCondition = (field) => ({
    field: field,
    op: "eq",
    value: null,
    id: uuid.v4()
});
const getDefaultGroup = () => ({
    connective: "and",
    id: uuid.v4()
});
const getSelectOption = (option) => {
    if (typeof option === "string") {
        return { value: option, label: option };
    }
    else if (typeof option === "number") {
        return { value: option.toString(), label: option.toString() };
    }
    else {
        return option;
    }
};
const getLocaleText = (key, locale) => locale !== undefined && locale[key] ? locale[key] : defaultLocale[key];
const deserialise = (obj) => {
    const [treeGroup, clauses] = groupObjToMap(obj, rootGroupUuid);
    return [
        Immutable__default["default"].Map({
            [rootGroupUuid]: treeGroup
        }),
        clauses
    ];
};
const groupObjToMap = (obj, id, clauses) => {
    let children = Immutable__default["default"].Map();
    if (!clauses) {
        clauses = Immutable__default["default"].Map();
    }
    clauses = clauses.set(id, Object.assign({ id: id }, obj));
    obj.children.forEach((child) => {
        const childId = uuid.v4();
        clauses = clauses.set(childId, Object.assign({ id: childId }, child));
        const g = child;
        if (g.connective) {
            const result = groupObjToMap(g, childId, clauses);
            children = children.set(childId, result[0]);
            clauses = clauses.merge(result[1]);
        }
        else {
            children = children.set(childId, childId);
        }
    });
    return [{ id: id, children: children }, clauses];
};
const escapeODataString = (val) => val.replace("'", "''");

const FilterInputs = ({ clauseId, field, onFieldChange, op, onOpChange, value, onValueChange, collectionOp, onCollectionOpChange, collectionField, onCollectionFieldChange }) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
    const schema = recoil.useRecoilValue(schemaState);
    const builderProps = recoil.useRecoilValue(propsState);
    const dateAdapter = React.useMemo(() => { var _a; return (_a = builderProps.localizationProviderProps) === null || _a === void 0 ? void 0 : _a.dateAdapter; }, [builderProps]);
    const fieldDef = React.useMemo(() => {
        var _a, _b, _c, _d, _e, _f, _g;
        if (!field && schema.length < 1) {
            return null;
        }
        let f;
        if (field) {
            f = (_a = schema.find(c => c.field === field)) !== null && _a !== void 0 ? _a : schema[0];
        }
        else {
            f = schema[0];
        }
        if (!f) {
            return null;
        }
        let filterField = field;
        let colField;
        let type = (_b = f.filterType) !== null && _b !== void 0 ? _b : f.type;
        let options = f.valueOptions;
        let ops = (_c = f.filterOperators) !== null && _c !== void 0 ? _c : allOperators;
        if (f.collection === true && f.collectionFields) {
            if (collectionField) {
                colField = (_d = f.collectionFields.find(c => c.field === collectionField)) !== null && _d !== void 0 ? _d : f.collectionFields[0];
            }
            else {
                colField = f.collectionFields[0];
            }
            filterField = colField.field;
            type = colField.type;
            options = colField.valueOptions;
            if (collectionOp !== "count") {
                ops = (_e = colField.filterOperators) !== null && _e !== void 0 ? _e : allOperators;
            }
            else {
                ops = numericOperators;
                type = "number";
            }
        }
        // get value options into a single type
        let valueOptions;
        if (type === "singleSelect" && typeof options === "function") {
            valueOptions = options({ field: filterField }).map((v) => getSelectOption(v));
        }
        else if (type === "singleSelect" && options) {
            valueOptions = options.map((v) => getSelectOption(v));
        }
        return Object.assign(Object.assign({}, f), { fieldLabel: (_g = (_f = f.label) !== null && _f !== void 0 ? _f : f.headerName) !== null && _g !== void 0 ? _g : f.field, type: type, ops: ops, valueOptions: valueOptions, colField: colField });
    }, [field, collectionField, collectionOp, schema]);
    const fieldOptions = React.useMemo(() => schema
        .filter(c => c.filterable !== false)
        .map(c => { var _a, _b, _c; return ({ label: (_b = (_a = c.label) !== null && _a !== void 0 ? _a : c.headerName) !== null && _b !== void 0 ? _b : c.field, field: c.field, group: (_c = c.autocompleteGroup) !== null && _c !== void 0 ? _c : "" }); })
        .sort((a, b) => builderProps.autocompleteGroups ?
        builderProps.autocompleteGroups.indexOf(a.group) - builderProps.autocompleteGroups.indexOf(b.group)
        : a.group.localeCompare(b.group)), [schema, builderProps]);
    if (schema.length < 1 || !fieldDef) {
        return null;
    }
    return (jsxRuntime.jsxs(React.Fragment, { children: [jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: fieldDef.collection ? true : 4 }, { children: jsxRuntime.jsx(material.Autocomplete, Object.assign({ size: "small" }, builderProps.autocompleteProps, { options: fieldOptions, renderInput: (params) => jsxRuntime.jsx(material.TextField, Object.assign({ label: getLocaleText("field", builderProps.localeText) }, builderProps.textFieldProps, params)), value: { label: fieldDef.fieldLabel, field: fieldDef.field, group: fieldDef.autocompleteGroup }, onChange: (_, val) => onFieldChange(fieldDef.field, op, val.field), disableClearable: true, isOptionEqualToValue: (option, value) => option.field === value.field, groupBy: (option) => option.group })) })), fieldDef.collection === true &&
                jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: true }, { children: jsxRuntime.jsxs(material.FormControl, Object.assign({ fullWidth: true, size: "small" }, { children: [jsxRuntime.jsx(material.InputLabel, Object.assign({ id: `${clauseId}_label-collection-op` }, { children: getLocaleText("collectionOperation", builderProps.localeText) })), jsxRuntime.jsxs(material.Select, Object.assign({ label: getLocaleText("collectionOperation", builderProps.localeText) }, builderProps.selectProps, { value: collectionOp, onChange: (e) => onCollectionOpChange(e.target.value), labelId: `${clauseId}_label-collection-op` }, { children: [jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "any" }, { children: getLocaleText("opAny", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "all" }, { children: getLocaleText("opAll", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "count" }, { children: getLocaleText("opCount", builderProps.localeText) }))] }))] })) })), fieldDef.collection === true && collectionOp !== "count" &&
                jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: true }, { children: jsxRuntime.jsx(material.Autocomplete, Object.assign({ size: "small" }, builderProps.autocompleteProps, { options: (_b = (_a = fieldDef.collectionFields) === null || _a === void 0 ? void 0 : _a.map(c => ({ label: c.label, field: c.field }))) !== null && _b !== void 0 ? _b : [], renderInput: (params) => jsxRuntime.jsx(material.TextField, Object.assign({ label: getLocaleText("collectionField", builderProps.localeText) }, builderProps.textFieldProps, params)), value: { label: (_c = fieldDef.colField) === null || _c === void 0 ? void 0 : _c.label, field: collectionField }, onChange: (_, val) => onCollectionFieldChange(field, collectionField, op, val.field), disableClearable: true, isOptionEqualToValue: (option, value) => option.field === value.field })) })), fieldDef.renderCustomFilter ?
                fieldDef.renderCustomFilter(value, onValueChange)
                :
                    jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: true }, { children: jsxRuntime.jsxs(material.FormControl, Object.assign({ fullWidth: true, size: "small" }, { children: [jsxRuntime.jsx(material.InputLabel, Object.assign({ id: `${clauseId}_label-op` }, { children: "Operation" })), jsxRuntime.jsxs(material.Select, Object.assign({}, builderProps.selectProps, { value: op, onChange: (e) => onOpChange(e.target.value), labelId: `${clauseId}_label-op`, label: "Operation" }, { children: [jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "eq", disabled: !fieldDef.ops.includes("eq") }, { children: getLocaleText("opEq", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "ne", disabled: !fieldDef.ops.includes("ne") }, { children: getLocaleText("opNe", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "gt", disabled: !fieldDef.ops.includes("gt") }, { children: getLocaleText("opGt", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "lt", disabled: !fieldDef.ops.includes("lt") }, { children: getLocaleText("opLt", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "ge", disabled: !fieldDef.ops.includes("ge") }, { children: getLocaleText("opGe", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "le", disabled: !fieldDef.ops.includes("le") }, { children: getLocaleText("opLe", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "contains", disabled: !fieldDef.ops.includes("contains") }, { children: getLocaleText("opContains", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "null", disabled: !fieldDef.ops.includes("null") }, { children: getLocaleText("opNull", builderProps.localeText) })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "notnull", disabled: !fieldDef.ops.includes("notnull") }, { children: getLocaleText("opNotNull", builderProps.localeText) }))] }))] })) })), !fieldDef.renderCustomFilter &&
                jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: true }, { children: op !== "null" && op !== "notnull" &&
                        (fieldDef.renderCustomInput ? fieldDef.renderCustomInput(value, onValueChange) :
                            jsxRuntime.jsxs(React.Fragment, { children: [fieldDef.type === "date" &&
                                        jsxRuntime.jsx(xDatePickers.LocalizationProvider, Object.assign({ dateAdapter: dateAdapter }, builderProps.localizationProviderProps, { children: jsxRuntime.jsx(xDatePickers.DatePicker, Object.assign({ label: getLocaleText("value", builderProps.localeText) }, builderProps.datePickerProps, fieldDef.datePickerProps, { value: value !== null && value !== void 0 ? value : "", renderInput: (params) => jsxRuntime.jsx(material.TextField, Object.assign({ fullWidth: true, size: "small" }, builderProps.textFieldProps, fieldDef.textFieldProps, params)), onChange: (date) => onValueChange(new dateAdapter().formatByString(date, "YYYY-MM-DD")) })) })), fieldDef.type === "datetime" &&
                                        jsxRuntime.jsx(xDatePickers.LocalizationProvider, Object.assign({ dateAdapter: dateAdapter }, builderProps.localizationProviderProps, { children: jsxRuntime.jsx(xDatePickers.DateTimePicker, Object.assign({ label: getLocaleText("value", builderProps.localeText) }, fieldDef.dateTimePickerProps, { value: value !== null && value !== void 0 ? value : "", renderInput: (params) => jsxRuntime.jsx(material.TextField, Object.assign({ fullWidth: true, size: "small" }, builderProps.textFieldProps, fieldDef.textFieldProps, params)), onChange: (date) => onValueChange(new dateAdapter().toISO(date)) })) })), fieldDef.type === "boolean" &&
                                        jsxRuntime.jsxs(material.FormControl, Object.assign({ fullWidth: true, size: "small" }, (_d = fieldDef.selectProps) === null || _d === void 0 ? void 0 : _d.formControlProps, { children: [jsxRuntime.jsx(material.InputLabel, Object.assign({ id: `${clauseId}_label-bool-value` }, { children: (_f = (_e = fieldDef.selectProps) === null || _e === void 0 ? void 0 : _e.label) !== null && _f !== void 0 ? _f : getLocaleText("value", builderProps.localeText) })), jsxRuntime.jsxs(material.Select, Object.assign({ label: (_h = (_g = fieldDef.selectProps) === null || _g === void 0 ? void 0 : _g.label) !== null && _h !== void 0 ? _h : getLocaleText("value", builderProps.localeText) }, builderProps.selectProps, (_j = fieldDef.selectProps) === null || _j === void 0 ? void 0 : _j.selectProps, { value: value !== null && value !== void 0 ? value : true, onChange: (e) => onValueChange(e.target.value), labelId: `${clauseId}_label-bool-value` }, { children: [jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "true" }, { children: "Yes" })), jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "false" }, { children: "No" })), fieldDef.nullable && jsxRuntime.jsx(material.MenuItem, Object.assign({ value: "null" }, { children: "Unknown" }))] }))] })), fieldDef.type === "singleSelect" && fieldDef.valueOptions &&
                                        jsxRuntime.jsxs(material.FormControl, Object.assign({ fullWidth: true, size: "small" }, (_k = fieldDef.selectProps) === null || _k === void 0 ? void 0 : _k.formControlProps, { children: [jsxRuntime.jsx(material.InputLabel, Object.assign({ id: `${clauseId}_label-select-value` }, { children: (_m = (_l = fieldDef.selectProps) === null || _l === void 0 ? void 0 : _l.label) !== null && _m !== void 0 ? _m : getLocaleText("value", builderProps.localeText) })), jsxRuntime.jsx(material.Select, Object.assign({ label: (_p = (_o = fieldDef.selectProps) === null || _o === void 0 ? void 0 : _o.label) !== null && _p !== void 0 ? _p : getLocaleText("value", builderProps.localeText), value: value !== null && value !== void 0 ? value : "", onChange: (e) => onValueChange(e.target.value), labelId: `${clauseId}_label-select-value` }, { children: fieldDef.valueOptions.map((o, i) => (jsxRuntime.jsx(material.MenuItem, Object.assign({ value: o.value }, { children: o.label }), `${clauseId}_${field}_select_${i}`))) }))] })), (!fieldDef.type || fieldDef.type === "string" || fieldDef.type === "number") &&
                                        jsxRuntime.jsx(material.TextField, Object.assign({ size: "small", fullWidth: true, label: getLocaleText("value", builderProps.localeText) }, builderProps.textFieldProps, fieldDef.textFieldProps, { value: value !== null && value !== void 0 ? value : "", onChange: (e) => onValueChange(fieldDef.type === "number" ? parseFloat(e.target.value) : e.target.value), type: fieldDef.type === "number" ? "number" : "text" }))] })) }))] }));
};
var FilterInputs$1 = React__default["default"].memo(FilterInputs);

const FilterCondition = ({ clauseId, path }) => {
    const [clauses, setClauses] = recoil.useRecoilState(clauseState);
    const setTree = recoil.useSetRecoilState(treeState);
    const schema = recoil.useRecoilValue(schemaState);
    const condition = React.useMemo(() => clauses.get(clauseId), [clauses, clauseId]);
    const changeField = React.useCallback((oldField, currentOp, newField) => {
        const oldFieldDef = schema.find(c => c.field === oldField);
        const newFieldDef = schema.find(c => c.field === newField);
        setClauses(old => old.update(clauseId, c => {
            var _a;
            const condition = Object.assign({}, c);
            condition.field = newField;
            condition.default = false;
            if (oldFieldDef && newFieldDef) {
                // reset value if fields have different types
                if (oldFieldDef.type !== newFieldDef.type) {
                    condition.value = "";
                }
                // reset operator if new field doesn't support current operator
                if (newFieldDef.filterOperators && !newFieldDef.filterOperators.includes(currentOp)) {
                    condition.op = (_a = newFieldDef.filterOperators[0]) !== null && _a !== void 0 ? _a : "eq";
                }
                // set collection field if new field is a collection
                if (newFieldDef.collection === true && newFieldDef.collectionFields) {
                    condition.collectionField = newFieldDef.collectionFields[0].field;
                    condition.collectionOp = "any";
                    if (newFieldDef.collectionFields[0].filterOperators) {
                        condition.op = newFieldDef.collectionFields[0].filterOperators[0];
                    }
                    else {
                        condition.op = "eq";
                    }
                }
                else { // clear collection fields if new field is not a collection
                    condition.collectionField = undefined;
                    condition.collectionOp = undefined;
                }
            }
            return condition;
        }));
    }, [schema, setClauses, clauseId]);
    const changeOp = React.useCallback((o) => {
        setClauses(old => old.update(clauseId, c => (Object.assign(Object.assign({}, c), { op: o, default: false }))));
    }, [setClauses, clauseId]);
    const changeValue = React.useCallback((v) => {
        setClauses(old => old.update(clauseId, c => (Object.assign(Object.assign({}, c), { value: v, default: false }))));
    }, [setClauses, clauseId]);
    const changeCollectionOp = React.useCallback((o) => {
        setClauses(old => old.update(clauseId, c => {
            const condition = Object.assign(Object.assign({}, c), { collectionOp: o, default: false });
            // reset field operator if switching to count operator and current op is not valid
            if (o === "count" && !numericOperators.includes(condition.op)) {
                condition.op = "eq";
            }
            return condition;
        }));
    }, [setClauses, clauseId]);
    const changeCollectionField = React.useCallback((field, oldColField, currentOp, newColField) => {
        const fieldDef = schema.find(c => c.field === field);
        setClauses(old => old.update(clauseId, c => {
            var _a;
            const condition = Object.assign({}, c);
            condition.collectionField = newColField;
            condition.default = false;
            if (fieldDef && fieldDef.collectionFields && oldColField && newColField) {
                const oldColFieldDef = fieldDef.collectionFields.find(c => c.field === oldColField);
                const newColFieldDef = fieldDef.collectionFields.find(c => c.field === newColField);
                // reset value if fields have different types
                if (oldColFieldDef.type !== newColFieldDef.type) {
                    condition.value = "";
                }
                // reset operator if new field doesn't support current operator
                if (newColFieldDef.filterOperators && !newColFieldDef.filterOperators.includes(currentOp)) {
                    condition.op = (_a = newColFieldDef.filterOperators[0]) !== null && _a !== void 0 ? _a : "eq";
                }
            }
            return condition;
        }));
    }, [schema, setClauses, clauseId]);
    const remove = React.useCallback(() => {
        // if not root group
        if (path.length > 2) {
            setTree(oldTree => oldTree.withMutations((old) => {
                // delete self
                old.deleteIn([...path, clauseId]);
                // get path to parent node (i.e. remove "children" from end of path)
                const parentPath = [...path];
                parentPath.splice(-1, 1);
                do {
                    const node = old.getIn(parentPath);
                    // delete parent if now empty
                    if (node && node.children.count() < 1) {
                        old.deleteIn(parentPath);
                    }
                    else { // not the only child, so only remove self and stop
                        old.deleteIn([...path, clauseId]);
                        break;
                    }
                    parentPath.splice(-2, 2); // move up in path to next parent
                } while (parentPath.length > 2); // keep removing empty groups until root is reached
            }));
        }
        else {
            setTree(old => old.deleteIn([...path, clauseId]));
        }
        setClauses(old => old.remove(clauseId));
    }, [setClauses, setTree, clauseId, path]);
    if (!condition) {
        return null;
    }
    return (jsxRuntime.jsxs(material.Grid, Object.assign({ container: true, spacing: 1 }, { children: [jsxRuntime.jsx(FilterInputs$1, { clauseId: clauseId, field: condition.field, onFieldChange: changeField, op: condition.op, onOpChange: changeOp, value: condition.value, onValueChange: changeValue, collectionOp: condition.collectionOp, onCollectionOpChange: changeCollectionOp, collectionField: condition.collectionField, onCollectionFieldChange: changeCollectionField }), jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: "auto" }, { children: jsxRuntime.jsx(material.IconButton, Object.assign({ onClick: remove }, { children: jsxRuntime.jsx(iconsMaterial.Remove, {}) })) }))] })));
};

const { makeStyles } = tssReact.createMakeStyles({ useTheme: material.useTheme });

const useStyles = makeStyles()((theme) => ({
    group: {
        borderWidth: 1,
        borderColor: theme.palette.mode === "dark" ? "rgb(81,81,81)" : "rgb(224,224,224)",
        borderRadius: theme.shape.borderRadius,
        borderStyle: "solid",
        padding: theme.spacing(2),
    },
    child: {
        position: "relative",
        "&:not(:last-of-type)::before": {
            content: "''",
            display: "block",
            position: "absolute",
            width: 2,
            height: "100%",
            background: theme.palette.primary.main,
            left: theme.spacing(-1),
        },
        "&:first-of-type::before": {
            height: `calc(100% + ${theme.spacing(2)})`,
            top: 0
        },
        "&::after": {
            content: "''",
            display: "block",
            position: "absolute",
            left: theme.spacing(-1),
            top: `calc(${theme.spacing(1)} + 1px)`,
            width: theme.spacing(2),
            height: "50%",
            borderWidth: 2,
            borderStyle: "solid",
            borderColor: theme.palette.primary.main,
            borderRight: "none",
            borderTop: "none",
            borderBottomLeftRadius: theme.shape.borderRadius
        }
    }
}));
const FilterGroup = ({ clauseId, path, root }) => {
    const { classes } = useStyles();
    const r = useResponsive();
    const [tree, clauses] = recoil.useRecoilValue(recoil.waitForAll([treeState, clauseState]));
    const setTree = recoil.useSetRecoilState(treeState);
    const setClauses = recoil.useSetRecoilState(clauseState);
    const schema = recoil.useRecoilValue(schemaState);
    const builderProps = recoil.useRecoilValue(propsState);
    const group = React.useMemo(() => clauses.get(clauseId), [clauses, clauseId]);
    const treeGroup = React.useMemo(() => tree.getIn([...path, clauseId]), [tree, path, clauseId]);
    const childrenPath = React.useMemo(() => [...path, clauseId, "children"], [path, clauseId]);
    const multiple = React.useMemo(() => treeGroup.children.count() > 1, [treeGroup]);
    const setConnective = React.useCallback((con) => {
        setClauses(clauses.update(clauseId, c => (Object.assign(Object.assign({}, c), { connective: con }))));
    }, [clauses, setClauses, clauseId]);
    const addGroup = React.useCallback(() => {
        const group = getDefaultGroup();
        const condition = getDefaultCondition(schema[0].field);
        setClauses(clauses
            .set(group.id, group)
            .set(condition.id, condition));
        setTree(tree
            .updateIn(childrenPath, (list) => list.set(group.id, { id: group.id, children: Immutable__default["default"].Map({ [condition.id]: condition.id }) })));
    }, [clauses, setClauses, tree, setTree, childrenPath, schema]);
    const addCondition = React.useCallback(() => {
        const condition = getDefaultCondition(schema[0].field);
        setClauses(clauses.set(condition.id, condition));
        setTree(tree
            .updateIn(childrenPath, (list) => list.set(condition.id, condition.id)));
    }, [clauses, setClauses, tree, setTree, childrenPath, schema]);
    const handleConnective = React.useCallback((event, val) => {
        if (val) {
            setConnective(val);
        }
    }, [setConnective]);
    return (jsxRuntime.jsxs(material.Grid, Object.assign({ item: !root, container: true, marginBottom: 1, paddingLeft: root ? 0 : 3, className: root ? "" : classes.group }, { children: [jsxRuntime.jsxs(material.Grid, Object.assign({ item: true, container: true, spacing: 1, justifyContent: multiple ? "space-between" : "end", alignItems: r({ xs: "flex-start", md: "center" }), marginBottom: 2, direction: r({ xs: "column-reverse", md: "row" }) }, { children: [multiple && (jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: "auto" }, { children: jsxRuntime.jsxs(material.ToggleButtonGroup, Object.assign({ value: group.connective, exclusive: true, onChange: handleConnective, color: "primary", "aria-label": `${getLocaleText("and", builderProps.localeText)}/${getLocaleText("or", builderProps.localeText)}`, size: "small" }, { children: [jsxRuntime.jsx(material.ToggleButton, Object.assign({ value: "and" }, { children: getLocaleText("and", builderProps.localeText) })), jsxRuntime.jsx(material.ToggleButton, Object.assign({ value: "or" }, { children: getLocaleText("or", builderProps.localeText) }))] })) }))), jsxRuntime.jsx(material.Grid, Object.assign({ item: true, xs: 12, md: "auto" }, { children: jsxRuntime.jsxs(material.ButtonGroup, Object.assign({ variant: "contained", size: "small", color: "secondary" }, { children: [jsxRuntime.jsx(material.Button, Object.assign({ startIcon: jsxRuntime.jsx(iconsMaterial.Add, {}), onClick: addCondition }, { children: getLocaleText("addCondition", builderProps.localeText) })), jsxRuntime.jsx(material.Button, Object.assign({ startIcon: jsxRuntime.jsx(iconsMaterial.Add, {}), onClick: addGroup }, { children: getLocaleText("addGroup", builderProps.localeText) }))] })) }))] })), jsxRuntime.jsx(material.Grid, Object.assign({ item: true, container: true, xs: true, direction: "column", rowSpacing: 2, columnSpacing: 1, paddingLeft: multiple ? 3 : 0 }, { children: treeGroup.children.toArray().map((c) => {
                    if (typeof c[1] === "string") {
                        return (jsxRuntime.jsx(material.Grid, Object.assign({ item: true, container: true, xs: true, className: multiple ? classes.child : "" }, { children: jsxRuntime.jsx(FilterCondition, { clauseId: c[0], path: childrenPath }) }), c[0]));
                    }
                    else {
                        return (jsxRuntime.jsx(material.Grid, Object.assign({ item: true, container: true, xs: true, className: multiple ? classes.child : "" }, { children: jsxRuntime.jsx(FilterGroup, { clauseId: c[0], path: childrenPath }) }), c[0]));
                    }
                }) }))] })));
};

const defaultTranslators = {
    "contains": ({ schema, field, value }) => {
        if ((schema.type && schema.type !== "string") || typeof value !== "string") {
            console.warn(`Warning: operation "contains" is only supported for fields of type "string"`);
            return false;
        }
        if (schema.caseSensitive === true) {
            return `contains(${field}, '${escapeODataString(value)}')`;
        }
        else {
            return `contains(tolower(${field}), tolower('${escapeODataString(value)}'))`;
        }
    },
    "null": ({ field }) => {
        return `${field} eq null`;
    },
    "notnull": ({ field }) => {
        return `${field} ne null`;
    },
    "default": ({ schema, field, op, value }) => {
        if (schema.type === "date") {
            return `date(${field}) ${op} ${value}`;
        }
        else if (schema.type === "datetime") {
            return `${field} ${op} ${value}`;
        }
        else if (schema.type === "boolean") {
            return `${field} ${op} ${value}`;
        }
        else if (!schema.type || schema.type === "string" || typeof value === "string") {
            if (schema.caseSensitive === true) {
                return `${field} ${op} '${escapeODataString(value)}'`;
            }
            else {
                return `tolower(${field}) ${op} tolower('${escapeODataString(value)}')`;
            }
        }
        else {
            return `${field} ${op} ${value}`;
        }
    }
};

const UseODataFilter = () => {
    const schema = recoil.useRecoilValue(schemaState);
    const [clauses, tree] = recoil.useRecoilValue(recoil.waitForAll([clauseState, treeState]));
    return React.useCallback(() => {
        return buildGroup(schema, clauses, tree, rootGroupUuid, []);
    }, [schema, clauses, tree]);
};
const UseODataFilterWithState = () => {
    const schema = recoil.useRecoilValue(schemaState);
    return React.useCallback((clauses, tree) => {
        return buildGroup(schema, clauses, tree, rootGroupUuid, []);
    }, [schema]);
};
const buildGroup = (schema, clauses, tree, id, path) => {
    const clause = clauses.get(id);
    const treeNode = tree.getIn([...path, id]);
    if (!treeNode) {
        console.error(`Tree node ${[...path, id].join("->")} not found`);
        return false;
    }
    const childClauses = treeNode.children
        .toArray()
        .map((c) => {
        if (typeof c[1] === "string") {
            return buildCondition(schema, clauses, c[0]);
        }
        else {
            return buildGroup(schema, clauses, tree, c[0], [...path, id, "children"]);
        }
    })
        .filter(c => c !== false);
    if (childClauses.length > 1) {
        return {
            filter: `(${childClauses.filter(c => c.filter).map(c => c.filter).join(` ${clause.connective} `)})`,
            compute: `${childClauses.filter(c => c.compute).map(c => c.compute).join(",")}`,
            select: childClauses.filter(c => c.select).flatMap(c => c.select),
            serialised: { connective: clause.connective, children: childClauses.map(c => c.serialised) },
            queryString: childClauses.reduce((x, c) => (Object.assign(Object.assign({}, x), c.queryString)), {})
        };
    }
    else if (childClauses.length === 1) {
        return {
            filter: childClauses[0].filter,
            compute: childClauses[0].compute,
            select: childClauses[0].select,
            serialised: { connective: clause.connective, children: [childClauses[0].serialised] },
            queryString: childClauses[0].queryString
        };
    }
    else {
        console.error("Group has no children");
        return false;
    }
};
const buildCondition = (schema, clauses, id) => {
    var _a;
    const clause = clauses.get(id);
    let condition = undefined;
    if (!clause || clause.default === true) {
        console.error(`Clause not found: ${id}`);
        return false;
    }
    else {
        condition = {
            field: clause.field,
            op: clause.op,
            collectionOp: clause.collectionOp,
            collectionField: clause.collectionField,
            value: clause.value
        };
    }
    const def = schema.find(d => d.field === clause.field);
    if (!def) {
        console.error(`Schema entry not found for field "${clause.field}"`);
        return false;
    }
    const filterField = (_a = def.filterField) !== null && _a !== void 0 ? _a : def.field;
    let innerResult;
    if (clause.collectionOp) {
        if (clause.collectionOp === "count") {
            innerResult = {
                filter: `${filterField}/$count ${clause.op} ${clause.value}`
            };
        }
        else {
            const collectionDef = def.collectionFields.find(d => d.field === clause.collectionField);
            innerResult = buildInnerCondition(collectionDef, "x/" + clause.collectionField, clause.op, clause.value);
        }
    }
    else {
        innerResult = buildInnerCondition(def, filterField, clause.op, clause.value);
    }
    if (typeof innerResult !== "boolean") {
        if (innerResult.filter) {
            return {
                filter: innerResult.filter,
                compute: innerResult.compute,
                select: innerResult.select,
                serialised: condition
            };
        }
        else {
            return {
                serialised: condition,
                queryString: innerResult.queryString
            };
        }
    }
    else {
        return false;
    }
};
const buildInnerCondition = (schema, field, op, value) => {
    if (schema.getCustomQueryString) {
        return {
            queryString: schema.getCustomQueryString(op, value)
        };
    }
    if (schema.getCustomFilterString) {
        const result = schema.getCustomFilterString(op, value);
        if (typeof result === "string") {
            return {
                filter: result
            };
        }
        else if (typeof result !== "boolean") {
            const compute = result.compute;
            if (typeof compute === "string") {
                return {
                    filter: result.filter,
                    compute: compute
                };
            }
            else {
                return {
                    filter: result.filter,
                    compute: compute.compute,
                    select: compute.select
                };
            }
        }
        else {
            return result;
        }
    }
    let translator;
    if (op in defaultTranslators) {
        translator = defaultTranslators[op];
    }
    else {
        translator = defaultTranslators["default"];
    }
    const result = translator({ schema, field, op, value });
    if (typeof result === "string") {
        return {
            filter: result
        };
    }
    else {
        return result;
    }
};

const FilterRoot = ({ props }) => {
    const setClauses = recoil.useSetRecoilState(clauseState);
    const setProps = recoil.useSetRecoilState(propsState);
    const setSchema = recoil.useSetRecoilState(schemaState);
    const setTree = recoil.useSetRecoilState(treeState);
    const odataFilter = UseODataFilter();
    const odataFilterWithState = UseODataFilterWithState();
    const [anchor, setAnchor] = React.useState(null);
    const { onSubmit, onRestoreState, disableHistory, filter: propsFilter } = props;
    const submit = React.useCallback((e) => {
        e.preventDefault();
        if (onSubmit) {
            const result = odataFilter();
            if (result.filter) {
                const returned = onSubmit(Object.assign(Object.assign({}, result), { filter: result.filter }));
                if (disableHistory !== true) {
                    window.history.pushState(Object.assign(Object.assign(Object.assign({}, window.history.state), returned), { filterBuilder: {
                            filter: result.filter,
                            compute: result.compute,
                            select: result.select,
                            serialised: result.serialised,
                            queryString: result.queryString
                        } }), "");
                }
            }
        }
    }, [onSubmit, odataFilter, disableHistory]);
    const reset = React.useCallback(() => {
        setClauses(initialClauses.update(rootConditionUuid, (c) => (Object.assign(Object.assign({}, c), { field: props.schema[0].field }))));
        setTree(initialTree);
        if (onSubmit) {
            onSubmit({ filter: "" });
        }
        if (disableHistory !== true) {
            window.history.pushState(Object.assign(Object.assign({}, window.history.state), { filterBuilder: {
                    reset: true
                } }), "");
        }
    }, [setClauses, setTree, onSubmit, props.schema, disableHistory]);
    const handleReset = React.useCallback(() => reset(), [reset]);
    React.useEffect(() => {
        setSchema(props.schema);
    }, [props.schema, setSchema]);
    const restoreDefault = React.useCallback(() => {
        setClauses(initialClauses.update(rootConditionUuid, (c) => (Object.assign(Object.assign({}, c), { field: props.schema[0].field }))));
        setTree(initialTree);
    }, [props.schema, setClauses, setTree]);
    const restoreState = React.useCallback((state, isPopstate) => {
        let filter = "", serialised, queryString, compute, select;
        if (state === null || state === void 0 ? void 0 : state.filterBuilder) {
            if (state.filterBuilder.reset === true && isPopstate === true) {
                restoreDefault();
            }
            compute = state.filterBuilder.compute;
            filter = state.filterBuilder.filter;
            select = state.filterBuilder.select;
            serialised = state.filterBuilder.serialised;
            queryString = state.filterBuilder.queryString;
        }
        else {
            restoreDefault();
        }
        if (filter && serialised) {
            const [tree, clauses] = deserialise(serialised);
            setClauses(clauses);
            setTree(tree);
        }
        if (onRestoreState) {
            onRestoreState({ compute, filter, queryString, select, serialised }, state);
        }
    }, [onRestoreState, restoreDefault, setClauses, setTree]);
    const restoreFilter = React.useCallback((serialised) => {
        var _a;
        const [tree, clauses] = deserialise(serialised);
        setClauses(clauses);
        setTree(tree);
        if (onRestoreState) {
            const result = odataFilterWithState(clauses, tree);
            onRestoreState(Object.assign(Object.assign({}, result), { filter: (_a = result.filter) !== null && _a !== void 0 ? _a : "" }));
        }
    }, [setClauses, setTree, onRestoreState, odataFilterWithState]);
    React.useEffect(() => {
        if (disableHistory !== true) {
            const handlePopState = (e) => { restoreState(e.state, true); };
            window.addEventListener("popstate", handlePopState);
            return () => window.removeEventListener("popstate", handlePopState);
        }
    }, [disableHistory, restoreState]);
    React.useEffect(() => {
        if (propsFilter) {
            restoreFilter(propsFilter);
        }
        else {
            restoreDefault();
        }
    }, [propsFilter, restoreFilter, restoreDefault]);
    useMountEffect(() => {
        setProps(props);
        // restore query from history state if enabled
        if (disableHistory !== true && window.history.state && window.history.state.filterBuilder) {
            restoreState(window.history.state, false);
        }
        else if (propsFilter) {
            restoreFilter(propsFilter);
        }
        else {
            restoreDefault();
        }
    });
    return (jsxRuntime.jsx(React.Fragment, { children: jsxRuntime.jsxs("form", Object.assign({ onSubmit: submit }, { children: [jsxRuntime.jsx(FilterGroup, { clauseId: rootGroupUuid, path: [], root: true }), jsxRuntime.jsxs(material.Grid, Object.assign({ container: true, spacing: 1 }, { children: [jsxRuntime.jsx(material.Grid, Object.assign({ item: true }, { children: jsxRuntime.jsxs(material.ButtonGroup, Object.assign({ variant: "contained", color: "primary" }, { children: [jsxRuntime.jsx(material.Button, Object.assign({ type: "submit" }, { children: "Search" })), props.searchMenuItems &&
                                        jsxRuntime.jsx(material.Button, Object.assign({ size: "small", onClick: (e) => setAnchor(e.currentTarget), "aria-controls": anchor !== null ? "search-menu" : undefined, "aria-expanded": anchor !== null ? "true" : undefined, "aria-haspopup": "menu" }, { children: jsxRuntime.jsx(iconsMaterial.ArrowDropDown, {}) }))] })) })), jsxRuntime.jsx(material.Grid, Object.assign({ item: true }, { children: jsxRuntime.jsx(material.Button, Object.assign({ variant: "outlined", onClick: handleReset }, { children: "Reset" })) }))] })), props.searchMenuItems &&
                    jsxRuntime.jsx(material.Popover, Object.assign({ anchorEl: anchor, open: anchor !== null, anchorOrigin: { vertical: "bottom", horizontal: "left" }, onClose: () => setAnchor(null), transitionDuration: 100 }, { children: jsxRuntime.jsx(material.Paper, { children: jsxRuntime.jsx(material.MenuList, Object.assign({ id: "search-menu" }, { children: props.searchMenuItems.map((item, i) => (jsxRuntime.jsx(material.MenuItem, Object.assign({ onClick: item.onClick }, { children: item.label }), `searchMenu_${i}`))) })) }) }))] })) }));
};

const FilterBuilder = (props) => {
    return (jsxRuntime.jsx(recoil.RecoilRoot, Object.assign({ override: true }, { children: jsxRuntime.jsx(FilterRoot, { props: props }) })));
};

const defaultPageSize = 10;

/**
 * Convert an Expand object (or array of objects) to a clause to use in an OData $expand query parameter
 * @param e Expand(s) to convert
 * @returns OData expand clause string
 */
const ExpandToQuery = (expand) => {
    if (expand === undefined) {
        return "";
    }
    if (!Array.isArray(expand)) {
        return ExpandToQuery([expand]);
    }
    // group all expands by the navigation field
    const groupedExpands = GroupArrayBy(expand, (e) => e.navigationField);
    // construct a single expand for each navigation field, combining nested query options (where possible)
    const expands = [];
    groupedExpands.forEach((e, k) => {
        var _a, _b;
        expands.push({
            navigationField: k,
            top: (_a = e.find(e2 => e2.top)) === null || _a === void 0 ? void 0 : _a.top,
            orderBy: (_b = e.find(e2 => e2.orderBy)) === null || _b === void 0 ? void 0 : _b.orderBy,
            count: e.some(e2 => e2.count),
            select: Array.from(new Set(e.filter(e2 => e2.select).map(e2 => e2.select))).join(","),
            expand: e.filter(e2 => e2.expand)
                .map(e2 => e2.expand)
                .reduce((a, b) => Array.isArray(b) ? a.concat(b) : [...a, b], [])
        });
    });
    return expands.map(e => {
        let result = `${e.navigationField}`;
        const options = [
            { type: "select", value: e.select },
            { type: "expand", value: ExpandToQuery(e.expand) },
            { type: "orderby", value: e.orderBy },
            { type: "top", value: e.top },
            { type: "count", value: e.count }
        ];
        if (options.some(o => o.value)) {
            result += `(${options.filter(o => o.value).map(o => `$${o.type}=${o.value}`).join(";")})`;
        }
        return result;
    }).join(",");
};
/**
 * Group an array into multiple arrays linked by a common key value
 * @param arr Array to group
 * @param keySelector Function to select property to group by
 * @returns ES6 Map of keys to arrays of values
 */
const GroupArrayBy = (arr, keySelector) => arr
    .reduce((m, e) => m.set(keySelector(e), [...m.get(keySelector(e)) || [], e]), new Map());
/**
 * Flatten an object to a single level, i.e. { Person: { Name: "John" } } becomes { "Person.Name": "John" }.
 * Arrays are kept as arrays, with their elements flattened.
 * @param obj Object to flatten
 * @param sep Level separator (default ".")
 * @returns Flattened object
 */
const Flatten = (obj, sep = ".") => _flatten(obj, sep, "");
const _flatten = (obj, sep, prefix) => Object.keys(obj).reduce((x, k) => {
    if (obj[k] !== null) {
        const pre = prefix.length ? prefix + sep : "";
        if (Array.isArray(obj[k])) {
            x[pre + k] = obj[k].map(i => Flatten(i, sep));
        }
        else if (typeof obj[k] === "object") {
            Object.assign(x, _flatten(obj[k], sep, pre + k));
        }
        else {
            x[pre + k] = obj[k];
        }
    }
    return x;
}, {});
const GetPageNumber = () => {
    const params = new URLSearchParams(window.location.search);
    if (params.has("page")) {
        const pageVal = params.get("page");
        if (pageVal) {
            return parseInt(pageVal, 10) - 1;
        }
    }
    return 0;
};
const GetPageSizeOrDefault = (defaultSize) => {
    const params = new URLSearchParams(window.location.search);
    if (params.has("page-size")) {
        const sizeVal = params.get("page-size");
        if (sizeVal) {
            return parseInt(sizeVal, 10);
        }
    }
    return defaultSize !== null && defaultSize !== void 0 ? defaultSize : defaultPageSize;
};

const ODataGridBase = (props) => {
    var _a;
    const [pageNumber, setPageNumber] = React.useState(GetPageNumber());
    const [pageSize, setPageSize] = React.useState(GetPageSizeOrDefault(props.defaultPageSize));
    const [rows, setRows] = React.useState([]);
    const [rowCount, setRowCount] = React.useState(0);
    const [loading, setLoading] = React.useState(true);
    const [sortModel, setSortModel] = React.useState(props.defaultSortModel);
    const [filter, setFilter] = React.useState("");
    const [filterSelects, setFilterSelects] = React.useState();
    const [compute, setCompute] = React.useState();
    const [queryString, setQueryString] = React.useState();
    const [visibleColumns, setVisibleColumns] = React.useState(props.columns
        .filter(c => (props.columnVisibilityModel && props.columnVisibilityModel[c.field] !== false) || c.hide !== true)
        .map(c => c.field));
    const [columnVisibilityOverride, setColumnVisibilityOverride] = React.useState({});
    const firstLoad = React.useRef(true);
    const fetchCount = React.useRef(true);
    const pendingFilter = React.useRef(false);
    const r = useResponsive();
    const fetchData = React.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        var _b;
        if (!filter
            && props.disableFilterBuilder !== true
            && ((_b = props.filterBuilderProps) === null || _b === void 0 ? void 0 : _b.disableHistory) !== true
            && window.history.state
            && window.history.state.filterBuilder
            && window.history.state.filterBuilder.reset !== true) {
            // stop fetch if there is no filter but there is one in history which will be/has been restored
            // this prevents a race condition between the initial data load and the query being restored
            return;
        }
        setLoading(true);
        // select all fields for visible columns
        const fields = new Set(props.columns
            .filter(c => visibleColumns.includes(c.field) && c.expand === undefined && c.filterOnly !== true && c.type !== "actions")
            .map(c => { var _a; return (_a = c.select) !== null && _a !== void 0 ? _a : c.field; }));
        if (props.alwaysSelect) {
            props.alwaysSelect.forEach((c) => fields.add(c));
        }
        if (filterSelects) {
            filterSelects.forEach((s) => fields.add(s));
        }
        const expands = props.columns
            .filter(c => visibleColumns.includes(c.field) && c.expand)
            .map(c => c.expand)
            .reduce((a, b) => Array.isArray(b) ? a.concat(b) : [...a, b], []);
        const query = new URLSearchParams();
        if (fields.size > 0) {
            query.append("$select", Array.from(fields).join(","));
        }
        if (expands.length > 0) {
            query.append("$expand", ExpandToQuery(expands));
        }
        query.append("$top", pageSize.toString());
        query.append("$skip", (pageNumber * pageSize).toString());
        if (fetchCount.current) {
            query.append("$count", "true");
        }
        if (queryString) {
            for (const key in queryString) {
                query.append(key, queryString[key]);
            }
        }
        if (filter) {
            query.append("$filter", filter);
        }
        else if (props.$filter) {
            query.append("$filter", props.$filter);
        }
        if (compute) {
            query.append("$compute", compute);
        }
        if (sortModel && sortModel.length > 0) {
            const sortCols = sortModel
                .map(s => ({ col: props.columns.find(c => c.field === s.field), sort: s.sort }))
                .filter(c => c.col)
                .map(c => { var _a; return `${(_a = c.col.sortField) !== null && _a !== void 0 ? _a : c.col.field}${c.sort === "desc" ? " desc" : ""}`; });
            if (sortCols.length > 0) {
                query.append("$orderby", sortCols.join(","));
            }
        }
        const response = yield fetch(props.url + "?" + query.toString(), props.requestOptions);
        if (response.ok) {
            const data = yield response.json();
            // flatten object so that the DataGrid can access all the properties
            // i.e. { Person: { name: "John" } } becomes { "Person/name": "John" }
            // keep the original object in the "result" property so that it can still be accessed via strong typing
            const rows = data.value.map((v) => (Object.assign({ result: v }, Flatten(v, "/"))));
            if (data["@odata.count"]) {
                setRowCount(data["@odata.count"]);
            }
            setRows(rows);
            setLoading(false);
            firstLoad.current = false;
            pendingFilter.current = false;
            fetchCount.current = false;
        }
        else {
            console.error(`API request failed: ${response.url}, HTTP ${response.status}`);
        }
    }), [
        pageNumber,
        pageSize,
        visibleColumns,
        sortModel,
        filter,
        filterSelects,
        compute,
        queryString,
        props.url,
        props.alwaysSelect,
        props.columns,
        props.$filter,
        props.disableFilterBuilder,
        (_a = props.filterBuilderProps) === null || _a === void 0 ? void 0 : _a.disableHistory,
        props.requestOptions
    ]);
    const handleBuilderSubmit = React.useCallback((params) => {
        var _a;
        pendingFilter.current = true;
        fetchCount.current = true;
        if ((_a = props.filterBuilderProps) === null || _a === void 0 ? void 0 : _a.onSubmit) {
            props.filterBuilderProps.onSubmit(params);
        }
        setCompute(params.compute);
        setFilter(params.filter);
        setFilterSelects(params.select);
        setQueryString(params.queryString);
        setPageNumber(0);
        return { oDataGrid: { sortModel: sortModel } };
    }, [props.filterBuilderProps, sortModel]);
    const handleBuilderRestore = React.useCallback((params, state) => {
        var _a, _b;
        fetchCount.current = true;
        if ((_a = props.filterBuilderProps) === null || _a === void 0 ? void 0 : _a.onRestoreState) {
            props.filterBuilderProps.onRestoreState(params, state);
        }
        if (props.disableHistory !== true) {
            if ((_b = state === null || state === void 0 ? void 0 : state.oDataGrid) === null || _b === void 0 ? void 0 : _b.sortModel) {
                setSortModel(state.oDataGrid.sortModel);
            }
            else {
                setSortModel(props.defaultSortModel);
            }
        }
        setCompute(params.compute);
        setFilter(params.filter);
        setFilterSelects(params.select);
        setQueryString(params.queryString);
    }, [props.filterBuilderProps, props.disableHistory, props.defaultSortModel]);
    React.useEffect(() => {
        fetchData();
    }, [fetchData]);
    const { onColumnVisibilityModelChange, onSortModelChange } = props;
    const handleSortModelChange = React.useCallback((model, details) => {
        if (onSortModelChange) {
            onSortModelChange(model, details);
        }
        setSortModel(model);
        if (props.disableHistory !== true) {
            window.history.pushState(Object.assign(Object.assign({}, window.history.state), { oDataGrid: { sortModel: model } }), "");
        }
    }, [onSortModelChange, props.disableHistory]);
    React.useEffect(() => {
        var _a, _b;
        let changed = false;
        const params = new URLSearchParams(window.location.search);
        // update page query string parameter
        const pageStr = params.get("page");
        if (pageStr) {
            const page = parseInt(pageStr, 10) - 1;
            // update if already exists and is different to settings
            if (page !== pageNumber) {
                if (pageNumber !== 0) {
                    params.set("page", (pageNumber + 1).toString());
                }
                else {
                    // remove if first page
                    params.delete("page");
                }
                changed = true;
            }
        }
        else if (pageNumber !== 0) {
            // add if doesn't already exist and not on first page
            params.set("page", (pageNumber + 1).toString());
            changed = true;
        }
        // update page-size query string parameter
        const sizeStr = params.get("page-size");
        if (sizeStr) {
            const size = parseInt(sizeStr, 10);
            if (size !== pageSize) {
                if (pageSize !== ((_a = props.defaultPageSize) !== null && _a !== void 0 ? _a : defaultPageSize)) {
                    params.set("page-size", pageSize.toString());
                }
                else {
                    params.delete("page-size");
                }
                changed = true;
            }
        }
        else if (pageSize !== ((_b = props.defaultPageSize) !== null && _b !== void 0 ? _b : defaultPageSize)) {
            params.set("page-size", pageSize.toString());
            changed = true;
        }
        // only run if modified and not the first load
        if (changed && !firstLoad.current) {
            const search = params.toString();
            const url = search ? `${window.location.pathname}?${search}${window.location.hash}` : `${window.location.pathname}${window.location.hash}`;
            // replace the state instead of pushing if a state has already been pushed by a filter
            if (pendingFilter.current) {
                window.history.replaceState(window.history.state, "", url);
            }
            else {
                window.history.pushState(window.history.state, "", url);
            }
        }
    }, [pageNumber, pageSize, props.defaultPageSize]);
    React.useEffect(() => {
        const handlePopState = (e) => {
            var _a, _b, _c, _d;
            const params = new URLSearchParams(window.location.search);
            const pageVal = params.get("page");
            if (pageVal) {
                const page = parseInt(pageVal, 10) - 1;
                setPageNumber(page);
            }
            else if (pageNumber !== 0) {
                // reset to first page if not provided and not already on first page
                setPageNumber(0);
            }
            const sizeVal = params.get("page-size");
            if (sizeVal) {
                const size = parseInt(sizeVal, 10) - 1;
                setPageSize(size);
            }
            else if ((_a = pageSize !== props.defaultPageSize) !== null && _a !== void 0 ? _a : defaultPageSize) {
                // reset to default if not provided and not already default
                setPageSize((_b = props.defaultPageSize) !== null && _b !== void 0 ? _b : defaultPageSize);
            }
            if (props.disableHistory !== true && props.disableFilterBuilder === true) {
                // only restore sort model from history if history is enabled and FilterBuilder is disabled
                // if FilterBuilder is enabled sort model restoration is handled in handleBuilderRestore
                if ((_d = (_c = e.state) === null || _c === void 0 ? void 0 : _c.oDataGrid) === null || _d === void 0 ? void 0 : _d.sortModel) {
                    setSortModel(e.state.oDataGrid.sortModel);
                }
                else {
                    setSortModel(props.defaultSortModel);
                }
            }
        };
        window.addEventListener("popstate", handlePopState);
        return () => window.removeEventListener("popstate", handlePopState);
    }, [pageNumber, pageSize, props.defaultPageSize, props.defaultSortModel, props.disableHistory, props.disableFilterBuilder]);
    const handlePageChange = React.useCallback((page) => {
        setPageNumber(page);
    }, []);
    const handlePageSizeChange = React.useCallback((size) => {
        setPageSize(size);
    }, []);
    const visibility = React.useMemo(() => {
        const v = {};
        if (props.columnVisibilityModel) {
            for (const field in props.columnVisibilityModel) {
                if (field in columnVisibilityOverride) {
                    v[field] = columnVisibilityOverride[field];
                }
                else if (typeof props.columnVisibilityModel[field] === "boolean") {
                    v[field] = props.columnVisibilityModel[field];
                }
                else {
                    v[field] = r(props.columnVisibilityModel[field]);
                }
            }
        }
        else {
            props.columns.filter(c => c.filterOnly !== true).forEach(c => {
                if (c.field in columnVisibilityOverride) {
                    v[c.field] = columnVisibilityOverride[c.field];
                }
                else if (typeof c.hide === "boolean") {
                    v[c.field] = !c.hide;
                }
                else if (c.hide) {
                    v[c.field] = !r(c.hide);
                }
            });
        }
        props.columns.filter(c => c.filterOnly === true).forEach(c => {
            v[c.field] = false;
        });
        return v;
    }, [props.columnVisibilityModel, r, props.columns, columnVisibilityOverride]);
    const handleColumnVisibilityModelChange = React.useCallback((model, details) => {
        if (onColumnVisibilityModelChange) {
            onColumnVisibilityModelChange(model, details);
        }
        // find the field which has been changed
        const column = Object.keys(model).find((key) => visibility[key] !== model[key]);
        if (column) {
            const visible = model[column];
            setColumnVisibilityOverride((v) => (Object.assign(Object.assign({}, v), { [column]: visible })));
            if (visible) {
                setVisibleColumns((v) => [...v, column]);
            }
            else {
                setVisibleColumns((v) => v.filter(c => c !== column));
            }
        }
    }, [onColumnVisibilityModelChange, visibility]);
    const gridColumns = React.useMemo(() => props.columns.filter(c => c.filterOnly !== true), [props.columns]);
    const GridComponent = props.component;
    return (jsxRuntime.jsxs(React.Fragment, { children: [props.$filter === undefined && props.disableFilterBuilder !== true &&
                jsxRuntime.jsx(system.Box, Object.assign({ mb: 2 }, { children: jsxRuntime.jsx(FilterBuilder, Object.assign({}, props.filterBuilderProps, { schema: props.columns, onSubmit: handleBuilderSubmit, onRestoreState: handleBuilderRestore })) })), jsxRuntime.jsx(GridComponent, Object.assign({ autoHeight: true, ref: React__default["default"].createRef() }, props, { columns: gridColumns, rows: rows, rowCount: rowCount, pagination: true, paginationMode: "server", page: pageNumber, pageSize: pageSize, onPageChange: handlePageChange, onPageSizeChange: handlePageSizeChange, disableColumnFilter: true, loading: loading, columnVisibilityModel: visibility, onColumnVisibilityModelChange: handleColumnVisibilityModelChange, sortingMode: "server", sortModel: sortModel, onSortModelChange: handleSortModelChange }))] }));
};

const ODataGridPro = (props) => (jsxRuntime.jsx(ODataGridBase, Object.assign({}, props, { component: xDataGridPro.DataGridPro })));

exports.FilterBuilder = FilterBuilder;
exports.ODataGridPro = ODataGridPro;
exports.allOperators = allOperators;
exports.defaultTranslators = defaultTranslators;
exports.escapeODataString = escapeODataString;
exports.numericOperators = numericOperators;
