/// <reference types="react" />
import { DataGridProProps, GridSortModel, GridColDef, GridActionsColDef, GridEnrichedColDef } from '@mui/x-data-grid-pro';
import { Breakpoint } from '@mui/material/styles';
import React from 'react';
import { DatePickerProps, DateTimePickerProps, LocalizationProviderProps } from '@mui/x-date-pickers';
import { AutocompleteProps, SelectProps, TextFieldProps, FormControlProps } from '@mui/material';
import { GridValueOptionsParams } from '@mui/x-data-grid';

declare type ResponsiveValues<P> = Partial<Record<Breakpoint, P>>;

declare type ExternalBuilderProps<TDate = any> = {
    searchMenuItems?: ({
        label: string;
        onClick: () => void;
    })[];
    onSubmit?: (params: FilterParameters) => (void | any);
    onRestoreState?: (params: FilterParameters, state?: any) => void;
    localeText?: FilterBuilderLocaleText;
    autocompleteGroups?: string[];
    autocompleteProps?: AutocompleteProps<any, any, any, any>;
    datePickerProps?: DatePickerProps<string, TDate>;
    dateTimePickerProps?: DateTimePickerProps<string, TDate>;
    localizationProviderProps?: LocalizationProviderProps;
    selectProps?: SelectProps;
    textFieldProps?: TextFieldProps;
    disableHistory?: boolean;
    filter?: SerialisedGroup;
};
declare type FilterParameters = {
    compute?: string;
    filter: string;
    queryString?: QueryStringCollection;
    select?: string[];
    serialised?: SerialisedGroup;
};
declare type FilterBuilderLocaleText = {
    and?: string;
    or?: string;
    addCondition?: string;
    addGroup?: string;
    field?: string;
    operation?: string;
    value?: string;
    collectionOperation?: string;
    collectionField?: string;
    search?: string;
    reset?: string;
    opAny?: string;
    opAll?: string;
    opCount?: string;
    opEq?: string;
    opNe?: string;
    opGt?: string;
    opLt?: string;
    opGe?: string;
    opLe?: string;
    opContains?: string;
    opNull?: string;
    opNotNull?: string;
};
declare type BaseFieldDef<TDate> = {
    field: string;
    autocompleteGroup?: string;
    caseSensitive?: boolean;
    datePickerProps?: DatePickerProps<string, TDate>;
    dateTimePickerProps?: DateTimePickerProps<string, TDate>;
    filterable?: boolean;
    filterField?: string;
    filterOperators?: Operation[];
    filterType?: string;
    getCustomFilterString?: (op: Operation, value: any) => string | FilterCompute | boolean;
    getCustomQueryString?: (op: Operation, value: any) => QueryStringCollection;
    label?: string;
    nullable?: boolean;
    selectProps?: {
        selectProps?: SelectProps;
        formControlProps?: FormControlProps;
        label?: string;
    };
    sortField?: string;
    textFieldProps?: TextFieldProps;
    renderCustomInput?: (value: any, setValue: (v: any) => void) => React.ReactNode;
    renderCustomFilter?: (value: any, setValue: (v: any) => void) => React.ReactNode;
    type?: string;
    valueOptions?: ValueOption[] | ((params: GridValueOptionsParams) => ValueOption[]);
};
declare type FieldDef<TDate = any> = BaseFieldDef<TDate> & {
    headerName?: string;
    collection?: boolean;
    collectionFields?: CollectionFieldDef<TDate>[];
};
declare type CollectionFieldDef<TDate = any> = BaseFieldDef<TDate>;
declare type FilterCompute = {
    filter: string;
    compute: string | ComputeSelect;
};
declare type ComputeSelect = {
    compute: string;
    select: string[];
};
declare type QueryStringCollection = {
    [key: string]: string;
};
declare type FilterTranslatorCollection<TDate> = {
    [key in Operation | "default"]?: FilterTranslator<TDate>;
};
declare type FilterTranslator<TDate> = (params: FilterTranslatorParams<TDate>) => string | boolean;
declare type FilterTranslatorParams<TDate> = {
    schema: BaseFieldDef<TDate>;
    field: string;
    op: Operation;
    value: any;
};
declare type Connective = "and" | "or";
declare type Operation = "eq" | "ne" | "gt" | "lt" | "ge" | "le" | "contains" | "null" | "notnull";
declare type CollectionOperation = "any" | "all" | "count";
declare type Clause = {
    id: string;
};
declare type GroupClause = Clause & {
    connective: Connective;
};
declare type ConditionClause = Clause & {
    field: string;
    op: Operation;
    collectionOp?: CollectionOperation;
    collectionField?: string;
    value: any;
    default?: boolean;
};
declare type SerialisedGroup = Omit<GroupClause, "id"> & {
    children: (SerialisedGroup | SerialisedCondition)[];
};
declare type SerialisedCondition = Omit<ConditionClause, "id" | "default">;

declare type ODataGridBaseProps<ComponentProps extends IGridProps, SortModel extends IGridSortModel, ColDef, TRow, TDate> = OmitGridProps<ComponentProps> & {
    url: string;
    alwaysSelect?: string[];
    columns: ODataGridBaseColDef<ColDef, TDate>[];
    columnVisibilityModel?: ODataColumnVisibilityModel;
    component: React.ElementType;
    defaultPageSize?: number;
    defaultSortModel?: SortModel;
    disableFilterBuilder?: boolean;
    disableHistory?: boolean;
    $filter?: string;
    filterBuilderProps?: ExternalBuilderProps<TDate>;
    requestOptions?: RequestInit;
};
declare type OmitGridProps<T> = Omit<T, "columns" | "rows" | "autoPageSize" | "columnVisibilityModel" | "disableColumnFilter" | "filterMode" | "filterModel" | "loading" | "onFilterModelChange" | "onPageChange" | "onPageSizeChange" | "page" | "pageSize" | "paginationMode" | "rowCount" | "sortingMode" | "sortModel">;
declare type ODataColumn<T, TDate> = Omit<T, "filterOperators" | "hide" | "sortComparator"> & FieldDef<TDate> & {
    select?: string;
    expand?: Expand | Expand[];
    hide?: ResponsiveValues<boolean> | boolean;
    filterOnly?: boolean;
};
declare type ODataRowModel<T> = {
    result: T;
    [key: string]: any;
};
declare type ODataGridBaseColDef<ColDef, TDate> = ODataColumn<ColDef, TDate>;
declare type ODataGridBaseEnrichedColDef<ColDef, ActionsColDef, TDate> = ODataColumn<ColDef, TDate> | ODataColumn<ActionsColDef, TDate>;
declare type ODataBaseGridColumns<EnrichedColDef, ActionsColDef, TDate> = ODataGridBaseEnrichedColDef<EnrichedColDef, ActionsColDef, TDate>[];
declare type Expand = {
    navigationField: string;
    select?: string;
    expand?: Expand[] | Expand;
    orderBy?: string;
    top?: number;
    count?: boolean;
};
declare type ValueOption = string | number | SelectOption;
declare type SelectOption = {
    value: any;
    label: string;
};
declare type ODataColumnVisibilityModel = Record<string, boolean | ResponsiveValues<boolean>>;
declare type ColumnVisibilityModel = Record<string, boolean>;
declare type IGridSortModel = ({
    field: string;
    sort: 'asc' | 'desc' | null | undefined;
})[];
declare type IGridProps = {
    onColumnVisibilityModelChange?: any;
    columnVisibilityModel?: ColumnVisibilityModel;
    onSortModelChange?: any;
};

declare type ODataGridProProps = Omit<ODataGridBaseProps<DataGridProProps, GridSortModel, GridColDef>, "component">;

declare const ODataGridPro: (props: ODataGridProProps) => JSX.Element;

declare type FilterBuilderProps<TDate = any> = ExternalBuilderProps<TDate> & {
    schema: FieldDef<TDate>[];
};
declare const FilterBuilder: <TDate>(props: FilterBuilderProps<TDate>) => JSX.Element;

declare const allOperators: Operation[];
declare const numericOperators: Operation[];

declare const escapeODataString: (val: string) => string;

declare const defaultTranslators: FilterTranslatorCollection<any>;

declare type ODataGridColDef<TRow = any, TDate = any> = ODataGridBaseColDef<GridColDef<ODataRowModel<TRow>>, TDate>;
declare type ODataGridEnrichedColDef<TRow = any, V = any, F = any, TDate = any> = ODataGridBaseEnrichedColDef<GridColDef<ODataRowModel<TRow>, V, F>, GridActionsColDef<ODataRowModel<TRow>>, TDate>;
declare type ODataGridColumns<TRow = any, TDate = any> = ODataBaseGridColumns<GridEnrichedColDef<ODataRowModel<TRow>>, GridActionsColDef<ODataRowModel<TRow>>, TDate>;

export { CollectionFieldDef, CollectionOperation, ComputeSelect, Connective, ExternalBuilderProps, FieldDef, FilterBuilder, FilterBuilderLocaleText, FilterBuilderProps, FilterCompute, FilterParameters, ODataColumnVisibilityModel, ODataGridColDef, ODataGridColumns, ODataGridEnrichedColDef, ODataGridPro, ODataGridProProps, QueryStringCollection, SelectOption, SerialisedCondition, SerialisedGroup, ValueOption, allOperators, defaultTranslators, escapeODataString, numericOperators };
